/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <stdlib.h>
#include <qdatastream.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include "patternfile.h"
#include "kxstitchdragobject.h"

PatternFile::PatternFile(QString path)
{
  m_read = false;
  m_changed = false;
  m_path = path;
}

PatternFile::~PatternFile()
{
  writeFile();
}

bool PatternFile::isWritable()
{
  return QFileInfo(m_path).isWritable();
}

void PatternFile::addPattern(LibraryPattern *pattern)
{
  m_patterns.append(pattern);
  m_changed = true;
  writeFile();
}

void PatternFile::addPattern(KXStitchDragObject *pattern)
{
  QByteArray data = pattern->encodedData("application/kxstitch");
  Q_UINT16 checksum = qChecksum(data.data(), data.size());
  addPattern(new LibraryPattern(0, Qt::NoButton, 0, checksum, data));
}

void PatternFile::deletePattern( LibraryPattern * pattern )
{
  m_patterns.removeRef(pattern);
  m_changed = true;
  writeFile();
}

LibraryPattern *PatternFile::first()
{
  if (!m_read)
    readFile();
  return m_patterns.first();
}

LibraryPattern *PatternFile::next()
{
  return m_patterns.next();
}

QString PatternFile::path()
{
  return m_path;
}

void PatternFile::readFile()
{
  if (!m_read)
  {
    bool ok = true;
    QFile file(localFile());
    if (file.open(IO_ReadOnly))
    {
      QDataStream stream(&file);
      char *header = (char *)malloc(11);
      stream.readRawBytes(header, 11);
      if (strncmp(header, "KXStitchLib", 11))
      {
        // probably an old library, try and read the old format
        file.reset();
        Q_UINT32 patterns;
        stream >> patterns;
        while (patterns-- && ok)
        {
          Q_UINT16 checksum;
          QByteArray data;
          QByteArray compressed;
          stream >> checksum;
          stream >> compressed;
          data = qUncompress(compressed);
          if (checksum == qChecksum(data.data(),data.size()))
          {
            m_patterns.append(new LibraryPattern(0, Qt::NoButton , 0, checksum, data));
          }
          else
          {
            KMessageBox::sorry(0, i18n("There was a checksum error in %1.").arg(localFile()), i18n("Checksum Error"));
            ok = false;
          }
        }
      }
      else
      {
        Q_UINT16 version;
        stream >> version;
        while (!file.atEnd() && ok)
        {
          Q_INT32 key;
          Q_INT32 modifier;
          Q_INT16 baseline;
          Q_UINT16 checksum;
          QByteArray data;
          stream >> key;
          stream >> modifier;
          stream >> baseline;
          stream >> checksum;
          stream >> data;
          if (checksum == qChecksum(data.data(), data.size()))
          {
            m_patterns.append(new LibraryPattern(key, (Qt::ButtonState)modifier, baseline, checksum, data));
          }
          else
          {
            KMessageBox::sorry(0, i18n("There was a checksum error in %1.").arg(localFile()), i18n("Checksum Error"));
            ok = false;
          }
        }
      }
      free(header);
      file.close();
      m_read = true;
    }
    else
    {
      // error message why file could not be opened
    }
  }
  m_changed = false;
}

void PatternFile::writeFile()
{
  if (hasChanged())
  {
    QFile file(localFile());
    if (file.open(IO_WriteOnly)) // truncates the file
    {
      QDataStream stream(&file);
      stream.writeRawBytes("KXStitchLib", 11);
      stream << (Q_UINT16)LIBVERSION;
      for (LibraryPattern *pattern = m_patterns.first() ; pattern ; pattern = m_patterns.next())
      {
        stream << pattern->key();
        stream << (Q_INT32)pattern->modifiers();
        stream << pattern->baseline();
        stream << pattern->checksum();
        stream << pattern->data();
      }
      file.close();
      m_read = true;
      m_changed = false;
    }
  }
}

QString PatternFile::localFile()
{
  QFileInfo path(m_path);
  if (path.isDir())
    return m_path+"kxstitch.library";
  else
    return m_path;
}

bool PatternFile::hasChanged()
{
  bool changed = m_changed;

  for (LibraryPattern *pattern = first() ; pattern ; pattern = next())
  {
    if (pattern->hasChanged())
      changed = true;
  }
  return changed;
}
